/***************************************************************************
 *
 * common.c : Common functions shared between modules
 *
 * Copyright (C) 2005 Richard Hughes, <richard@hughsie.com>
 *
 * Licensed under the GNU GPL v2.  See COPYING.
 *
 **************************************************************************/

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <glib.h>
#include <gdk/gdkx.h>
#include <gnome.h>
#include <gconf/gconf-client.h>

#include "common.h"
#include "logger.h"

/** Updates .minutesRemaining in a GenericObject
 *  This function is needed because of the different ways that batteries can represent their time remaining.
 *
 *  @param  slotData		the GenericObject reference
 *  @param  datavalue 		Data value, discharge/charge rate
 */
void
updateMinutesRemainingFromSlot (GenericObject *slotData)
{
	GPM_TRACE (("updateMinutesRemainingFromSlot"));
	if (slotData->powerDevice == POWER_NONE)
		return;
	if (slotData->reportType == REPORT_MWH) {
		/* we need to calculate for acpi-style */
		slotData->minutesRemaining = 0;
		if (slotData->rawRate>0) {
			if (slotData->isCharging)
				/* we are charge rate */
				slotData->minutesRemaining = ((double) (slotData->rawLastFull-slotData->rawCharge) / (double) slotData->rawRate) * 60;
			else
				/* we are discharge rate */
				slotData->minutesRemaining = ((double) slotData->rawCharge / (double) slotData->rawRate) * 60;
		}
	} else if (slotData->reportType == REPORT_PERCENT) {
		/* percent style is already done for us */
		slotData->minutesRemaining = slotData->rawMinutesRemaining;
	} else
		slotData->minutesRemaining = 0;
}

/** Updates .percentageCharge in a GenericObject
 *  This function is needed because of the different ways that batteries can represent their charge.
 *
 *  @param  slotData		the GenericObject reference
 *  @param  datavalue 		Data value, either percentage or new mWh current
 */
void
updatePercentageChargeFromSlot (GenericObject *slotData)
{
	GPM_TRACE (("updatePercentageChargeFromSlot"));
	if (!slotData) {
		GPM_ERROR (("updatePercentageChargeFromSlot : slotData is NULL"));
		return;
	}
	if (slotData->powerDevice == POWER_NONE)
		return;
	if (slotData->powerDevice==POWER_AC_ADAPTER || 
	    slotData->powerDevice==POWER_UNKNOWN) {
		/* cannot query, assume 100% */
		slotData->percentageCharge = 100;
		return;
	}

	if (slotData->rawLastFull <= 0)
		slotData->rawLastFull = 100; /* shouldn't happen */

	/* work out the ACTUAL percentage charge of the battery using cached values */
	slotData->percentageCharge = ((double) slotData->rawCharge / (double) slotData->rawLastFull) * 100;

	/* make sure results are sensible */
	if (slotData->percentageCharge < 0)
		slotData->percentageCharge = 0;
	if (slotData->percentageCharge > 100)
		slotData->percentageCharge = 100;
}

/** Returns the time string, e.g. "2 hours 3 minutes until charged"
 *
 *  @param  slotData		the GenericObject reference
 *  @param  tooltip		The timestring 
 */
void
getChargeTimeString (GenericObject *slotData, char *timestring)
{
	GPM_TRACE (("getChargeTimeString"));
	if (!slotData) {
		GPM_ERROR (("getChargeTimeString : slotData is NULL"));
		return;
	}
	int hours, minutes;				
	/* get minutes remaining */
	if (slotData->percentageCharge<1) {
		sprintf (timestring, "\nEmpty!");
		return;
	}
	if (slotData->percentageCharge>98) {
		/* report capacity, (how trucked your battery really is) */
		if (slotData->reportType == REPORT_MWH && slotData->rawLastFull > 0) {
			int capacity = ((double) slotData->rawDesign / (double) slotData->rawLastFull) * 100;
			sprintf (timestring, "\nFully Charged (%i%% capacity)", capacity);
		} else
			sprintf (timestring, "\nFully Charged");
		return;
	}
	if (slotData->minutesRemaining==0) {
		sprintf (timestring, "\nUnknown time remaining");
		return;
	}

	/* factor out minutes/seconds */
	if (slotData->minutesRemaining<60)
		sprintf (timestring, "\n%i minutes", slotData->minutesRemaining);
	else {
		hours = slotData->minutesRemaining / 60;
		minutes = slotData->minutesRemaining - (hours * 60);
		sprintf (timestring, "\n%i hours %i minutes", hours, minutes);
	}
	/* get suffix */
	if (slotData->isCharging)
		strcat (timestring, " until charged");
	else
		strcat (timestring, " remaining");
}

/** Returns the tooltip for the icon. Text logic goes here :-)
 *
 *  @param  slotData		the GenericObject reference
 *  @param  tooltip		The tooltip
 */
void
getTooltipFromSlot (GenericObject *slotData, char *tooltip)
{
	GPM_TRACE (("getTooltipFromSlot"));
	if (!slotData) {
		GPM_ERROR (("getTooltipFromSlot : slotData is NULL"));
		return;
	}
	char remaining[128];

	if (slotData->powerDevice == POWER_NONE)
		return;
	if (slotData->powerDevice == POWER_BATTERY) {
		getChargeTimeString (slotData, remaining);
		sprintf (tooltip, "%s Battery: %i%% %s", slotData->technologyType, slotData->percentageCharge, remaining);
	} else if (slotData->powerDevice == POWER_AC_ADAPTER) {
		sprintf (tooltip, "AC Adapter: Supplying power");
	} else if (slotData->powerDevice == POWER_UPS) {
		getChargeTimeString (slotData, remaining);
		sprintf (tooltip, "APC UPS: %i%% %s", slotData->percentageCharge, remaining);
	} else if (slotData->powerDevice == POWER_MOUSE) {
		sprintf (tooltip, "%s: %i%%", slotData->technologyType, slotData->percentageCharge);
	} else if (slotData->powerDevice == POWER_KEYBOARD) {
		sprintf (tooltip, "%s Keyboard: %i%%", slotData->technologyType, slotData->percentageCharge);
	} else {
		sprintf (tooltip, "Unknown Device");
	}
}

/** Updates .powerState in a GenericObject
 *
 *  @param  slotData		the GenericObject reference
 */
void
updatePowerStateFromSlot (GenericObject *slotData)
{
	GPM_TRACE (("updatePowerStateFromSlot"));
	if (!slotData) {
		GPM_ERROR (("updatePowerStateFromSlot : slotData is NULL"));
		return;
	}
	if (slotData->powerDevice == POWER_NONE)
		return;
	if (slotData->percentageCharge <= 10)
		slotData->powerState = CHARGE_EMPTY;
	else if (slotData->percentageCharge > 10 && slotData->percentageCharge <= 30)
		slotData->powerState = CHARGE_VLOW;
	else if (slotData->percentageCharge > 30 && slotData->percentageCharge <= 50)
		slotData->powerState = CHARGE_LOW;
	else if (slotData->percentageCharge > 50 && slotData->percentageCharge <= 70)
		slotData->powerState = CHARGE_HIGH;
	else if (slotData->percentageCharge > 70 && slotData->percentageCharge <= 99)
		slotData->powerState = CHARGE_VHIGH;
	else if (slotData->percentageCharge > 99)
		slotData->powerState = CHARGE_FULL;
}

/** Returns the icon filename of the icon. Picture logic goes here :-)
 *
 *  @param  number		The name of the filename (numeric part)
 *  @param  powerState		The power state, e.g. VLOW
 */
static void
getFilenameNumberFormat(char *number, int powerState)
{
	GPM_TRACE (("getFilenameNumberFormat"));
	if (powerState==CHARGE_EMPTY)
		strcpy (number, "000");
	else if (powerState==CHARGE_VLOW)
		strcpy (number, "020");
	else if (powerState==CHARGE_LOW)
		strcpy (number, "040");
	else if (powerState==CHARGE_HIGH)
		strcpy (number, "060");
	else if (powerState==CHARGE_VHIGH)
		strcpy (number, "080");
	else if (powerState==CHARGE_FULL)
		strcpy (number, "100");
}

/** Returns the icon filename of the icon. Picture logic goes here :-)
 *
 *  @param  slotData		the GenericObject reference
 *  @param  filename		Where to put the data
 */
void
getIconFilenameFromSlot (GenericObject *slotData, char *filename)
{
	GPM_TRACE (("getIconFilenameFromSlot"));
	if (!slotData) {
		GPM_ERROR (("getIconFilenameFromSlot : slotData is NULL"));
		return;
	}
	char number[4];
	if (slotData->powerDevice == POWER_NONE)
		return;
	if (slotData->powerDevice == POWER_BATTERY) {
		getFilenameNumberFormat(number, slotData->powerState);
		sprintf (filename, "battery-24-%s.png", number);
		return;
	} else if (slotData->powerDevice == POWER_AC_ADAPTER) {
		sprintf (filename, "ac_adapter-24.png");
		return;
	} else if (slotData->powerDevice == POWER_UPS) {
		getFilenameNumberFormat(number, slotData->powerState);
		sprintf (filename, "ups-24-%s.png", number);
		return;
	} else if (slotData->powerDevice == POWER_MOUSE) {
		getFilenameNumberFormat(number, slotData->powerState);
		sprintf (filename, "mouse-24-%s.png", number);
		return;
	} else if (slotData->powerDevice == POWER_KEYBOARD) {
		getFilenameNumberFormat(number, slotData->powerState);
		sprintf (filename, "keyboard-24-%s.png", number);
		return;
	}
}
